var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import simpleheat from 'simpleheat';
import { debounce } from '../utils/debounce';
import { finder } from '@medv/finder';
export class AnalyzeeHeatmaps {
    constructor(analyzee, options, log) {
        this._analyzee = analyzee;
        this._config = options;
        this.log = log;
        this._init().catch(() => {
            this.log('Failed to initialize heatmaps', 'error');
        });
    }
    destroy() {
    }
    startLoading() {
        var _a;
        (_a = this.loadingElement) === null || _a === void 0 ? void 0 : _a.remove();
        this.loadingElement = document.createElement('div');
        this.loadingElement.style.position = 'fixed';
        this.loadingElement.style.zIndex = '2147483647';
        this.loadingElement.style.backgroundColor = 'rgba(0, 0, 0, 0.5)';
        this.loadingElement.style.pointerEvents = 'none';
        this.loadingElement.style.top = '0px';
        this.loadingElement.style.left = '0px';
        this.loadingElement.style.width = '100%';
        this.loadingElement.style.height = '100%';
        this.loadingElement.style.display = 'flex';
        this.loadingElement.style.justifyContent = 'center';
        this.loadingElement.style.alignItems = 'center';
        this.loadingElement.innerHTML = '<div class="analyzee-heatmap-spinner"></div>';
        document.body.appendChild(this.loadingElement);
        if (this.tooltipElement) {
            this.tooltipElement.style.display = 'none';
        }
    }
    endLoading() {
        var _a;
        (_a = this.loadingElement) === null || _a === void 0 ? void 0 : _a.remove();
        if (this.tooltipElement) {
            this.tooltipElement.style.display = 'block';
        }
        this.buildCanvasElement();
    }
    buildCanvasElement() {
        var _a;
        (_a = this.canvasElement) === null || _a === void 0 ? void 0 : _a.remove();
        this.canvasElement = document.createElement('canvas');
        this.canvasElement.style.position = 'absolute';
        this.canvasElement.style.zIndex = '2147483646';
        this.canvasElement.style.pointerEvents = 'none';
        this.canvasElement.style.top = '0px';
        this.canvasElement.style.left = '0px';
        this.canvasElement.width = document.querySelector('html').offsetWidth;
        this.canvasElement.height = document.querySelector('html').offsetHeight;
        document.body.appendChild(this.canvasElement);
        const canvas = HTMLCanvasElement.prototype.transferControlToOffscreen ? this.canvasElement.transferControlToOffscreen() : this.canvasElement;
        this.heatmapInstance = simpleheat(canvas);
    }
    initHeatmaps() {
        window.addEventListener('mousemove', debounce((event) => {
            const hoverElement = document.elementFromPoint(event.clientX, event.clientY);
            if (!hoverElement) {
                return;
            }
            window.parent.postMessage({
                action: 'analyzee:heatmaps:mousemove',
                x: event.clientX,
                y: event.clientY,
                element: hoverElement.tagName
            }, '*');
            const target = event.target;
            if (this.lastHoveredElement === target || !target || !target.tagName || target.tagName === 'HTML' || target.tagName === 'BODY') {
                return;
            }
            this.lastHoveredElement = target;
            const selector = this.lastHoveredElement.analyzee_selector || finder(this.lastHoveredElement, {
                /**
                 * Minimum length for optimising selector.
                 * Starts from 2. For example selector
                 * body > div > div > p can be optimised to body p.
                 */
                optimizedMinLength: 3,
                maxNumberOfTries: 5000
            });
            this.lastHoveredElement.analyzee_selector = selector;
            this.renderTooltip(selector);
        }, 50, {
            maxWait: 150
        }), {
            passive: true
        });
        window.addEventListener('scroll', (event) => {
            window.parent.postMessage({
                action: 'analyzee:heatmaps:scroll',
                scrollY: window.scrollY,
                scrollHeight: document.body.scrollHeight,
                clientHeight: document.documentElement.clientHeight
            }, '*');
        }, {
            passive: true
        });
        window.addEventListener('resize', () => {
            window.parent.postMessage({
                action: 'analyzee:heatmaps:resize',
                scrollY: window.scrollY,
                scrollHeight: document.body.scrollHeight,
                clientHeight: document.documentElement.clientHeight
            }, '*');
        }, {
            passive: true
        });
        window.addEventListener('click', (event) => {
            window.parent.postMessage({
                action: 'analyzee:heatmaps:click',
                x: event.clientX,
                y: event.clientY,
            }, '*');
        }, {
            passive: true
        });
        window.parent.postMessage({
            action: 'analyzee:heatmaps:ready',
            scrollY: window.scrollY,
            scrollHeight: document.body.scrollHeight,
            clientHeight: document.documentElement.clientHeight,
        }, '*');
    }
    renderTooltip(selector) {
        var _a;
        if (!this.tooltipElement || !this.aggregatedClicks || !this.lastHoveredElement || !(selector || ((_a = this.lastHoveredElement) === null || _a === void 0 ? void 0 : _a.analyzee_selector))) {
            return;
        }
        const clicks = this.aggregatedClicks[selector || this.lastHoveredElement.analyzee_selector] || 0;
        const percentage = (Math.round((clicks / this.totalClicks) * 10000) / 100) || 0;
        this.tooltipElement.innerHTML = `<span style="font-weight: 600;">${clicks.toLocaleString()} clicks</span> <span style="font-weight: 500;">(${percentage}%)</span>`;
        const rect = this.lastHoveredElement.getBoundingClientRect();
        this.box.style.transform = `translate(${rect.left - 6}px, ${rect.top - 6}px)`;
        this.box.style.width = `${rect.width + 12}px`;
        this.box.style.height = `${rect.height + 12}px`;
        const tooltipTop = Math.max(6, rect.top - 10 - 48);
        const tooltipLeft = Math.min(Math.max(6, rect.left), window.innerWidth - 6 - 150);
        this.tooltipElement.style.transform = `translate(${tooltipLeft}px, ${tooltipTop}px)`;
    }
    renderHeatmap() {
        if (!this.heatmapInstance || !this.elementPositions) {
            return;
        }
        this.heatmapInstance.clear();
        const data = this.elementPositions.reduce((acc, click) => {
            var _a, _b;
            try {
                const escapedSelector = click.selector.replace(/:/g, '\\:');
                const clickElement = document.querySelector(escapedSelector);
                if (!clickElement || !click.relative) {
                    return acc;
                }
                clickElement.analyzee_selector = click.selector;
                const elementRect = clickElement.getBoundingClientRect();
                acc.push({
                    x: elementRect.left + ((_a = click === null || click === void 0 ? void 0 : click.relative) === null || _a === void 0 ? void 0 : _a.x),
                    y: elementRect.top + ((_b = click === null || click === void 0 ? void 0 : click.relative) === null || _b === void 0 ? void 0 : _b.y) + window.scrollY,
                    selector: clickElement.analyzee_selector
                });
            }
            catch (e) {
                // ignore
            }
            return acc;
        }, []);
        this.aggregatedClicks = this.calculateClicks(data);
        this.totalClicks = this.calculateTotalClicks(data) || 0;
        let max = 1;
        const hovered = {};
        for (const position of data) {
            this.heatmapInstance.add([position.x, position.y, 1]);
            const key = `${position.x}-${position.y}`;
            hovered[key] = (hovered[key] || 0) + 1;
            if (hovered[key] > max) {
                max = hovered[key];
                this.heatmapInstance.max(max);
            }
        }
        this.heatmapInstance.gradient({ 0.4: '#3c94be', 0.65: '#34d985', 0.85: '#e1de1c', 1: '#ff0000' });
        this.heatmapInstance.radius(15, 10);
        this.heatmapInstance.draw(0.1);
    }
    calculateClicks(data) {
        const regularClickStats = {};
        for (const item of data) {
            if (!regularClickStats[item.selector]) {
                regularClickStats[item.selector] = 0;
            }
            regularClickStats[item.selector]++;
        }
        const aggregatedClickStats = {};
        for (const selector in regularClickStats) {
            const selectorElement = document.querySelector(selector);
            if (!selectorElement) {
                continue;
            }
            const children = Array.from(document.querySelectorAll('*')).reduce((acc, child) => {
                if (!child.analyzee_selector) {
                    return acc;
                }
                if (child.analyzee_selector === selector || selectorElement.contains(child)) {
                    return [...acc, child.analyzee_selector];
                }
                return acc;
            }, []);
            aggregatedClickStats[selector] = children.reduce((acc, child) => {
                return acc + regularClickStats[child];
            }, 0);
        }
        return aggregatedClickStats;
    }
    calculateTotalClicks(data) {
        const selectors = Array.from(new Set(data.map((item) => item.selector)));
        const existingSelectors = selectors.filter((selector) => document.querySelector(selector));
        return data.reduce((acc, item) => {
            if (existingSelectors.includes(item.selector)) {
                return acc + 1;
            }
            return acc;
        }, 0);
    }
    _init() {
        return __awaiter(this, void 0, void 0, function* () {
            window.parent.postMessage({
                action: 'analyzee:heatmap:init:start'
            }, '*');
            this.startLoading();
            this.box = document.createElement('div');
            this.box.style.position = 'fixed';
            this.box.style.zIndex = '2147483647';
            this.box.style.padding = '10px';
            this.box.style.background = 'rgba(255, 100, 0, 0.3)';
            this.box.style.border = '2px dashed rgba(255, 100, 0, 0.5)';
            this.box.style.color = 'white';
            this.box.style.pointerEvents = 'none';
            this.box.style.transition = 'all 0.15s ease-in-out';
            this.box.style.top = '0px';
            this.box.style.left = '0px';
            this.box.style.width = '0px';
            this.box.style.height = '0px';
            this.box.style.borderRadius = '4px';
            document.body.appendChild(this.box);
            this.tooltipElement = document.createElement('div');
            this.tooltipElement.style.position = 'fixed';
            this.tooltipElement.style.zIndex = '2147483647';
            this.tooltipElement.style.padding = '10px';
            this.tooltipElement.style.pointerEvents = 'none';
            this.tooltipElement.style.backgroundColor = 'white';
            this.tooltipElement.style.border = '2px solid rgba(255, 100, 0, 0.5)';
            this.tooltipElement.style.borderRadius = '4px';
            this.tooltipElement.style.transition = 'all 0.15s ease-in-out';
            this.tooltipElement.style.top = '0px';
            this.tooltipElement.style.left = '0px';
            this.tooltipElement.style.display = 'none';
            document.body.appendChild(this.tooltipElement);
            this.buildCanvasElement();
            const messageListener = (event) => {
                if (event.data.action === 'analyzee:heatmaps:init') {
                    this.initHeatmaps();
                }
                else if (event.data.action === 'analyzee:heatmap:get:element:positions') {
                    this.elementPositions = event.data.payload || [];
                    this.renderHeatmap();
                    this.endLoading();
                }
                else if (event.data.action === 'analyzee:heatmap:loading') {
                    this.startLoading();
                }
            };
            window.addEventListener('message', messageListener);
            const heatmapChange = debounce(() => {
                this.renderHeatmap();
                this.renderTooltip();
            }, 100);
            window.addEventListener('scroll', heatmapChange);
            window.addEventListener('click', heatmapChange);
            setInterval(() => {
                heatmapChange();
            }, 1000);
            const resizeListener = debounce(() => {
                this.canvasElement.remove();
                this.buildCanvasElement();
                this.renderHeatmap();
                this.renderTooltip();
            }, 100);
            window.addEventListener('resize', resizeListener);
            const style = document.createElement('style');
            style.innerHTML = `
    .analyzee-heatmap-spinner {
        width: 48px;
        height: 48px;
        border: 5px solid #F69B38;
        border-bottom-color: transparent;
        border-radius: 50%;
        display: inline-block;
        box-sizing: border-box;
        animation: rotation 1s linear infinite;
    }

    @keyframes rotation {
        0% {
            transform: rotate(0deg);
        }
        100% {
            transform: rotate(360deg);
        }
    }
    `;
            document.head.appendChild(style);
            this.initHeatmaps();
        });
    }
}
