import { AnalyzeeEventType } from './../shared/interfaces/analyzee-event.interface';
export class AnalyzeeHotLeads {
    constructor(analyzee, _addInternalEvent, options, websocket) {
        /**
         * @private Last HotLeads click event
         */
        this._lastClickEvent = Date.now();
        this._analyzee = analyzee;
        this._addInternalEvent = _addInternalEvent;
        this._websocket = websocket;
        this._options = options;
        this._init();
    }
    showDiscount(options) {
        /**
         * TODO: Convert to iframe
         */
        const discountElement = document.createElement('div');
        discountElement.innerHTML = `<div id="hotleads-discount" style="position: fixed; top: 0; left: 0; right: 0; bottom: 0; z-index: 99999999; background-color: rgba(100, 100, 100, 0.6); display: flex; justify-content: center; align-items: center; color: white;">
  <div style="max-width: 90%; width: 480px; background-color: white; color: black; border-radius: 16px; font-family: Arial, sans-serif; padding: 12px 32px; position: relative;">
    
    <div style="position: absolute; top: 20px; right: 20px; cursor: pointer;" onclick="javascript:document.getElementById('hotleads-discount').remove();">
      <svg style="width: 30px; height: 30px;" class="w-6 h-6" fill="currentColor" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" d="M4.293 4.293a1 1 0 011.414 0L10 8.586l4.293-4.293a1 1 0 111.414 1.414L11.414 10l4.293 4.293a1 1 0 01-1.414 1.414L10 11.414l-4.293 4.293a1 1 0 01-1.414-1.414L8.586 10 4.293 5.707a1 1 0 010-1.414z" clip-rule="evenodd"></path></svg>
    </div>
    
    <h2 id="hotleads-discount-title" style="font-family: Sen, Roboto, SansSerif, sans-serif; font-size: 1.1em;">${options.title}</h2>
    <p id="hotleads-discount-description" style="font-family: Sen, Roboto, SansSerif, sans-serif; font-size: 0.9em; margin-top: 12px; margin-bottom: 12px;">
    ${options.description}
    </p>
    
    <div style="display: flex; height: 42px; max-height: 42px;">
      <div style="position: absolute; width: 40px; height: 42px; display: flex; justify-content: center; align-items: center;">
        <svg style="width: 20px; height: 20px;" class="w-6 h-6" fill="#60957c" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" d="M17.707 9.293a1 1 0 010 1.414l-7 7a1 1 0 01-1.414 0l-7-7A.997.997 0 012 10V5a3 3 0 013-3h5c.256 0 .512.098.707.293l7 7zM5 6a1 1 0 100-2 1 1 0 000 2z" clip-rule="evenodd"></path></svg>
      </div>
      <input id="hotleads-discount-code" type="text" readonly value="${options.code}" style="color: grey; padding: 0; margin: 0; font-size: 20px; line-height: 30px; width: 70%; height: 100%; border: 1px solid rgba(0, 0, 0, 0.5); border-radius: 8px; border-top-right-radius: 0; border-bottom-right-radius: 0; padding-left: 40px;">
      <button style="width: 30%; height: 100%; border: 0 solid black; border-left: none; border-radius: 8px; border-top-left-radius: 0; border-bottom-left-radius: 0; background-color: #e5653d; color: white; font-weight: bold; cursor: pointer;" onclick="javascript:navigator.clipboard.writeText(document.getElementById('hotleads-discount-code').value);document.getElementById('hotleads-discount-notification-message').style.opacity = '1';">
        Copy
      </button>
    </div>
    <div id="hotleads-discount-notification-message" style="font-size: 0.8em; color: #60957c; margin-top: 6px; opacity: 0; transition: opacity 0.5s;">Copied to clipboard</div>
  </div>`;
        document.body.appendChild(discountElement);
    }
    destroy() {
        clearInterval(this._hotLeadsPredictionInterval);
    }
    _init() {
        const handleClickEvent = (event) => {
            const target = event.target;
            const tag = target === null || target === void 0 ? void 0 : target.tagName;
            if (document.createElement(tag).constructor.name !== 'HTMLUnknownElement') {
                const tags = ['A', 'SPAN', 'BUTTON', 'SELECT', 'INPUT', 'TEXTAREA', 'IMG', 'IFRAME', 'VIDEO', 'AUDIO', 'EMBED', 'OBJECT', 'CANVAS'];
                if (!tags.includes(tag)) {
                    const levels = 3;
                    let current = target;
                    let hasValidParent = false;
                    for (let i = 0; i < levels; i++) {
                        if (hasValidParent) {
                            continue;
                        }
                        current = current === null || current === void 0 ? void 0 : current.parentElement;
                        hasValidParent = tags.includes(current === null || current === void 0 ? void 0 : current.tagName);
                    }
                    if (!hasValidParent) {
                        return;
                    }
                }
            }
            if (Date.now() - this._lastClickEvent < 1000) {
                return;
            }
            this._lastClickEvent = Date.now();
            const clicks = this._analyzee.extends.utilities.sessionGet('clicks') || 0;
            this._analyzee.extends.utilities.sessionSet('clicks', clicks + 1);
        };
        /**
         * TODO: Remove listeners on destroy
         */
        window.addEventListener('touchstart', handleClickEvent);
        window.addEventListener('click', handleClickEvent);
        this._analyzee.extends.utilities.sessionSet('createdOn', Date.now());
        this._hotLeadsPredictionInterval = setInterval(() => {
            const date = new Date();
            const data = {
                clicks: this._analyzee.extends.utilities.sessionGet('clicks') || 0,
                duration: (date.getTime() - (this._analyzee.extends.utilities.sessionGet('createdOn') || date.getTime())) / 1000,
            };
            this._addInternalEvent({
                type: AnalyzeeEventType.HotLeads,
                payload: data
            });
        }, 5 * 1000);
    }
}
