// import type {NetworkRequest} from 'rrweb';
// import {getRecordConsolePlugin, getRecordNetworkPlugin, record} from 'rrweb';
import { getRecordConsolePlugin, record } from 'rrweb';
import { AnalyzeeEventType } from './../shared/interfaces/analyzee-event.interface';
export class AnalyzeeSessions {
    constructor(analyzee, _addInternalEvent, config) {
        this._analyzee = analyzee;
        this._addInternalEvent = _addInternalEvent;
        this._config = config;
        this._init();
        /**
         * TODO:
         * In case we want manually take page snapshots, we can use
         * record.takeFullSnapshot(true);
         */
    }
    destroy() {
        clearInterval(this._snapshotInterval);
        this._snapshotInterval = null;
        if (typeof this._stopRecording === 'function') {
            this._stopRecording();
        }
    }
    /**
     * Takes a snapshot of the current page including all the DOM elements
     */
    takeSnapshot() {
        return record.takeFullSnapshot(true);
    }
    _init() {
        const plugins = [
            getRecordConsolePlugin({
                level: ['info', 'log', 'warn', 'error']
            })
        ];
        // if (this._config.recordNetwork) {
        //   plugins.push(
        //     getRecordNetworkPlugin({
        //       ignoreRequestFn: (request: NetworkRequest) => {
        //         /**
        //          * TODO: Check if request or response body is too big
        //          */
        //         this._addInternalEvent({
        //           type: AnalyzeeEventType.NetworkRequest,
        //           payload: {
        //             url: request.url,
        //             initiatorType: request.initiatorType,
        //             method: (request.method || 'GET') as AnalyzeeNetworkRequestEvent['payload']['method'],
        //             /**
        //              * https://developer.mozilla.org/en-US/docs/Web/API/PerformanceResourceTiming/responseStatus
        //              *
        //              * Firefox and Safari don't support responseStatus property
        //              */
        //             status: request.status === undefined ? 200 : (request.status === 0 ? 200 : request.status),
        //             startTime: request.startTime,
        //             endTime: request.endTime,
        //             requestBody: request.requestBody ?? undefined,
        //             requestHeaders: request.requestHeaders ?? undefined,
        //             responseBody: request.responseBody ?? undefined,
        //             responseHeaders: request.responseHeaders ?? undefined
        //           }
        //         });
        //
        //         return true;
        //       },
        //       recordHeaders: true,
        //       recordBody: true,
        //       recordInitialRequests: true
        //     })
        //   );
        // }
        const recordOptions = {
            emit: (event) => {
                this._addInternalEvent({
                    type: AnalyzeeEventType.SessionReplay,
                    payload: event
                });
            },
            blockClass: 'analyzee-block',
            ignoreClass: 'analyzee-ignore',
            maskTextClass: 'analyzee-mask',
            collectFonts: this._config.collectFonts,
            recordCanvas: this._config.recordCanvas,
            sampling: {
                input: 'all',
                media: 500,
                scroll: this._config.debounceScroll,
                mousemove: this._config.debounceMouseMove,
                mouseInteraction: true
            },
            plugins,
            // recordCrossOriginIframes: true
        };
        if (typeof this._config.maskInputs === 'boolean') {
            recordOptions.maskAllInputs = this._config.maskInputs;
        }
        else if (typeof this._config.maskInputs === 'object') {
            recordOptions.maskInputOptions = this._config.maskInputs;
        }
        this._stopRecording = record(recordOptions);
        this._snapshotInterval = setInterval(() => {
            this.takeSnapshot();
        }, this._config.snapshotInterval || 120000);
    }
}
